/*
 * Copyright (C) 2022 luca0N!
 *
 * This file is part of Static Website Generator (swg).
 *
 * Static Website Generator (swg) is free software: you can redistribute it
 * and/or modify it under the terms of the version 3 of the GNU Lesser General
 * Public License as published by the Free Software Foundation.
 *
 * Static Website Generator (swg) is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Static Website Generator (swg). If not, see
 * <https://www.gnu.org/licenses/>.
 *
 * Contact luca0N! by e-mail: <luca0n [at] luca0n [dot] com>. 
 */

#include <iostream>
#include <cstring>
#include <string>
#include <list>
#include <regex>

#include <filesystem>

#include "SwgContext.hxx"
#include "ConfigUtils.hxx"
// #include "Blog.h"
#include "Common.hxx"
#include "WebsiteBuilder.hxx"

bool verbose = false;

void printUsage(const char *programName) {
	std::cerr << programName << ": usage: " << programName << " [options] [path]"
		<< std::endl;
}

void print_help(const char *program_name) {
	printUsage(program_name);
	std::cout << " -h, --help\t\tShows this help message.\n"
			  << " -v, --verbose\t\tPrints additional messages to standard output.\n";
}

void printIntro() {
	printf("%s v%s (%d)\nCopyright (C) %s %s\n\n",
			PROGRAM_NAME, PROGRAM_VERSION_NAME, PROGRAM_VERSION,
			PROGRAM_COPYRIGHT_YEARS, PROGRAM_COPYRIGHT);
}

int main(int argc, char *argv[]) {
	printIntro();

	std::string path = "";

	// Parse cmdline options
	for (int argi = 1; argi < argc; argi++) {
		if (argv[argi][0] == '-') {
			// This is a cmdline option (starts with a dash).
			if (argv[argi][1] == '-') {
				// Long cmdline option (--example).
				if (strcmp(argv[argi], "--verbose") == 0) {
					verbose = true;
					std::cout << "Verbose output enabled.\n";
				} else if (strcmp(argv[argi], "--help") == 0) {
					print_help(argv[0]);
					exit(RETURN_SUCCESSFUL);
				} else {
					std::cerr << "error: unknown command line option `" <<
						argv[argi] << "'." << std::endl;
					exit(RETURN_FAILED_INVALID_SYNTAX);
				}
			} else {
				for (int i = 1; i < strlen(argv[argi]); i++) {
					switch (argv[argi][i]) {
						case 'v':
							verbose = true;
							std::cout << "Verbose output enabled.\n";
							break;
						case 'h':
							print_help(argv[0]);
							exit(RETURN_SUCCESSFUL);
							break;
						default:
							std::cerr << "error: unknown command line option `-" <<
								argv[argi][i] << "'." << std::endl;
							exit(RETURN_FAILED_INVALID_SYNTAX);
							break;
					}
				}
			}
		} else if (path == "") {
			path = argv[argi];
		} else {
			std::cerr << "error: website directory was already " <<
				"provided (`" << path << "'), refusing to continue with `" <<
				argv[argi] << "'." << std::endl;
			printUsage(argv[0]);
			exit(RETURN_FAILED_INVALID_SYNTAX);
		}
	}

	std::cout << "Using specified directory: " << path << "\n";

	SwgContext ctx;
	parse_config(&ctx, path);

	build_website(ctx, path);

	std::cout << "Website build succeeded.\n";

	// Website blog

	// Append blog to website
	//ctx.appendBlog(blog1);

	// Generate website
	//ctx.generateWebsite();

	return 0;
}
